; array.lsp

(defun make-array (dim &rest b)
    (let ((contents nil) (element nil) (type nil) (arr nil))
        (setq contents (get-contents b))
	(setq element (get-element b))
	(setq type (get-type b))
	(if (eql type 'character)
	    (cond
		((characterp element) t)
		((eql element 'nothing?) (setq element #\ ))
		(t nil) )
	    (progn (if (eql element 'nothing?) (setq element nil)) (setq type 'all)) )
	(cond
	    ((and (eql type 'character) (not (characterp element))) (print '(require character initial element)) nil)
	    ((and (integerp dim) (not (< dim 0)))
	     (setq arr (set-array (cons 1 (cons type (consn dim element)))))
             (set-contents arr (list dim) contents element) arr )
	    ((accept-dimension-element dim)
	     (setq arr (set-array (cons (length dim) (cons type (make-array-base (reverse dim) element)))))
	     (set-contents arr dim contents element) arr )
	    (t (print '(syntax error)) nil) ) ) )

(defun set-contents (arr dim contents def)
    (let ((pos (init-pos (length dim))) (el nil))
	(loop
	    (if (eql (car pos) (car dim)) (return t))
	    (setq el (a-content contents pos def))
	    (eval `(setf (aref arr ,@pos) el))
	    (setq pos (reverse (inc-pos (reverse pos) (reverse dim)))) ) ) )

(defun a-content (contents pos def)
    (let ((k))
	(setq k (car pos))
	(cond
	    ((atom contents) def)
	    ((>= k (length contents)) def)
	    ((atom (cdr pos)) (nth k contents))
	    (t (a-content (nth k contents) (cdr pos) def)) ) ) )

(defun inc-pos (pos dim)
    (let ((k))
	(setq k (1+ (car pos)))
	(cond
	    ((eql k (car dim))
	     (cond
		((atom (cdr pos)) (list k))
		(t (cons 0 (inc-pos (cdr pos) (cdr dim)))) ) )
	    (t
	     (cons k (cdr pos)) ) ) ) )

(defun init-pos (len)
    (cond
	((zerop len) nil)
	(t (cons 0 (init-pos (1- len)))) ) )

(defun make-array-base (dim element)
    (cond
	((atom dim) element)
	(t (make-array-base (cdr dim) (consn (car dim) element))) ) )

(defun copy (s)
    (cond
	((consp s) (cons (copy (car s)) (copy (cdr s))))
	(t s) ) )

(defun consn (n element)
    (cond
	((zerop n) nil)
	(t (cons element (consn (1- n) (copy element)))) ) )

(defun get-contents (b)
    (let ((f (find :initial-contents b)))
	(cond
	    ((null f) nil)
	    ((atom (cdr f)) nil)
	    (t (car (cdr f))) ) ) )

(defun get-element (b)
    (let ((f (find :initial-element b)))
	(cond
	    ((null f) 'nothing?)
	    ((atom (cdr f)) 'nothing?)
	    (t (car (cdr f))) ) ) )

(defun get-type (b)
    (let ((f (find :element-type b)))
	(cond
	    ((null f) nil)
	    ((atom (cdr f)) nil)
	    ((eql (car (cdr f)) 'character) 'character)
	    (t 'all) ) ) )

(defun find (v l)
    (cond
	((atom l) nil)
	((eql (car l) v) l)
	(t (find v (cdr l))) ) )

(defun accept-dimension-element (l)
    (cond
	((atom l) nil)
	(t (accept-dimension-element-base l)) ) )

(defun accept-dimension-element-base (l)
    (cond
	((atom l) t)
	((not (integerp (car l))) nil)
	((< (car l) 0) nil)
	(t (accept-dimension-element-base (cdr l))) ) )

(defun aref (a &rest b)
    (let ((dim (array-car a)))
	(cond
	    ((not (eql dim (length b))) (print '(error at setting dimension)) nil)
	    (t (get-array-pos (cdr (array-cdr a)) b)) ) ) )

(defun get-array-pos (a b)
    (cond
	((atom b) a)
	((not (integerp (car b))) (print '(error at setting dimension number)) nil)
	(t (get-array-pos (nth (car b) a) (cdr b))) ) )

(defun pre-aref (c a &rest b)
    (let ((dim (array-car a)) (type (car (array-cdr a))))
	(cond
	    ((not (eql dim (length b))) (print '(error at setting dimension)) nil)
	    ((and (eql type 'character) (not (characterp c))) (print '(error at setting character)) nil)
	    (t (pre-get-array-pos (cdr (array-cdr a)) b)) ) ) )

(defun pre-get-array-pos (a b)
    (cond
	((atom b) a)
	((not (integerp (car b))) (print '(error at setting dimension number)) nil)
	((eql (length b) 1) (nthcdr (car b) a))
	(t (pre-get-array-pos (nth (car b) a) (cdr b))) ) )

;(defmacro array-rplaca (x? y?) ; x? = (pre-aref arr ..)
;    (let ((type? (car (array-cdr (eval (cadr x?))))) (y1? (eval y?)))
;	(cond
;	    ((and (eql type? 'character) (not (characterp y1?))) (print '(require character)) nil)
;	    (t `(rplaca ,x? ,y?)) ) ) )

